﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NationalInstruments;
using NationalInstruments.DAQmx;

namespace Tuc.Control
{
    /// <summary>
    /// Reading and Writing Data from DAQ Device
    /// </summary>
    public class DaqData
    {
        
        public string aiChannel;
        public string aoChannel;


        //Constructor
        public DaqData(string aiChannel_in, string aoChannel_in)
        {

            aiChannel = aiChannel_in; //Example: "dev1/ai0"
            aoChannel = aoChannel_in; //Example: "dev1/ao0"

        }


        public double ReadDaqData()
        {
           
           Task analogInTask = new Task();

            AIChannel myAIChannel;
            myAIChannel = analogInTask.AIChannels.CreateVoltageChannel(
                aiChannel,
                "myAIChannel",
                AITerminalConfiguration.Differential,
                0,
                5,
                AIVoltageUnits.Volts
                );

            AnalogSingleChannelReader reader = new AnalogSingleChannelReader(analogInTask.Stream);

            double analogDataIn = reader.ReadSingleSample();

            return analogDataIn;           
           
        }


        public void WriteDaqData(double analogDataOut)
        {

           
            Task analogOutTask = new Task();

            AOChannel myAOChannel;

            try
            {

                myAOChannel = analogOutTask.AOChannels.CreateVoltageChannel(
                    aoChannel,
                    "myAOChannel",
                    0,
                    5,
                    AOVoltageUnits.Volts
                    );

                AnalogSingleChannelWriter writer = new AnalogSingleChannelWriter(analogOutTask.Stream);

                writer.WriteSingleSample(true, analogDataOut);
            }
            catch (Exception e)
            {
                string errorMessage;

                errorMessage = e.Message.ToString();
 
            }

            finally
            {
                analogOutTask.Stop();
            }

        }
    }
}
