﻿using NationalInstruments;
using NationalInstruments.UI;
using NationalInstruments.UI.WindowsForms;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Tuc.Control;
using System.Windows.Forms.DataVisualization.Charting;



namespace Control_Application
{
    public partial class Form1 : Form
    {

        // Initialization--------------------------------
        # region 
        
               
        //PID Controller
        PidController pidControl = new PidController 
            {
                Ts=0.1,
                r=5,
                Kp=0.8,
                Ti=15
            };


        //LowPass Filter
        //Filter filter = new Filter(0.1, 2); //Constructur
        Filter filter = new Filter
            {
                Ts = 0.1,
                Tf = 2
            };
        

        //Water Tank Model
        ProcessModel model = new ProcessModel(0.1);

        // DAQ
        DaqData myDaqData = new DaqData("dev1/ai0", "dev1/ao0");


        double levelMeasurement;
        double controllerOutput;

        
        public Form1()
        {
            InitializeComponent();

            timer1.Start();

            comboOperationMode.SelectedIndex = 0;


            chartMeasurementData.Series.Clear();
            chartMeasurementData.Series.Add("MeasurementData");
            chartMeasurementData.Series["MeasurementData"].ChartType = SeriesChartType.Line;

            chartMeasurementData.Series.Add("ReferenceData");
            chartMeasurementData.Series["ReferenceData"].ChartType = SeriesChartType.Line;


            ChartArea area1 = chartMeasurementData.ChartAreas[0];
            area1.AxisY.Minimum = 0;
            area1.AxisY.Maximum = 20;


            chartController.Series.Clear();
            chartController.Series.Add("ControllerData");
            chartController.Series["ControllerData"].ChartType = SeriesChartType.Line;

            ChartArea area2 = chartController.ChartAreas[0];
            area2.AxisY.Minimum = 0;
            area2.AxisY.Maximum = 5;

        }
        

        private void timer1_Tick(object sender, EventArgs e)
        {

            ProcessData();

            ControlSystem();
            
            Visualization();                               

        }


        # endregion


        // Private Functions-----------------------------
        # region 

               
        
        /// <summary>
        /// 
        /// </summary>
        private void ProcessData()
        {

            //Read Data 


            if (switchModelSelector.Value == true) //Use Real Process
            {

                levelMeasurement = myDaqData.ReadDaqData(); //Read from DAQ

                // Lowpass filtering the Measure Value due to noise
                levelMeasurement = filter.LowPassFilter(levelMeasurement);


                //Chech boundaries
                if (levelMeasurement < 0)
                    levelMeasurement = 0;
                if (levelMeasurement > 5)
                    levelMeasurement = 5;

                //Scaling:
                levelMeasurement = levelMeasurement * 4; //0-5V -> 0-20cm

            }
            else //Use Matematical Model
            {

                levelMeasurement = model.LevelTankModel(controllerOutput);

            }
            

            //Show Data in GUI
            txtLevelValue.Text = levelMeasurement.ToString("0.00");  

        }    


        
        /// <summary>
        /// 
        /// </summary>
        private void ControlSystem()
        {

            //Write Control Value            
            
            if (switchController.Value == true) //Use Manual Control
            {

                controllerOutput = Convert.ToDouble(numManualControl.Value);

            }
            else // Use PID Control
            {

                controllerOutput = pidControl.PiController(levelMeasurement);

                //Scaling
                controllerOutput = controllerOutput / 4; //0-20cm -> 0-5V

                //Set boundaries
                if (controllerOutput < 0)
                    controllerOutput = 0;
                if (controllerOutput > 5)
                    controllerOutput = 5;


            }

            myDaqData.WriteDaqData(controllerOutput); //Write to DAQ


            //Show Data in GUI
            txtControllerValue.Text = controllerOutput.ToString("0.00");


        }


        /// <summary>
        /// 
        /// </summary>
        private void Visualization()
        {
            
            //Plot Data
           chartMeasurementData.Series["MeasurementData"].Points.AddY(levelMeasurement);
           chartMeasurementData.Series["ReferenceData"].Points.AddY(Convert.ToDouble(txtSetPoint.Text));

           chartController.Series["ControllerData"].Points.AddY(controllerOutput);

        }


        #endregion



        // Events----------------------------------------
        # region 

        private void btnExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }



        private void Form1_Load(object sender, EventArgs e)
        {

        }



        private void switchController_StateChanged(object sender, ActionEventArgs e)
        {
            
            if (switchController.Value==false)
                numManualControl.Enabled = false;
            else
                numManualControl.Enabled = true;
            
        }
       

        private void txtKp_TextChanged(object sender, EventArgs e)
        {

            try
            {

                pidControl.Kp = Convert.ToDouble(txtKp.Text);
            }
            catch (Exception ex)
            {
                
                string errorMessage;

                errorMessage = "Please use correct Format.\n\n" + ex.ToString();

                MessageBox.Show(errorMessage);
            }


        }



        private void txtTi_TextChanged(object sender, EventArgs e)
        {

            pidControl.Ti = Convert.ToDouble(txtTi.Text);

        }



        private void txtSetPoint_TextChanged(object sender, EventArgs e)
        {

            double value;

            
            try
            {

                value = Convert.ToDouble(txtSetPoint.Text);
                if (value > 20)
                {
                    value = 20;
                    txtSetPoint.Text = "20";
                }
                if (value < 0)
                {
                    value = 0;
                    txtSetPoint.Text = "0";
                }

                pidControl.r = value;


            }
            catch (Exception ex)
            {

                string errorMessage;

                errorMessage = "Please use correct Format.\n\n" + ex.ToString();

                MessageBox.Show(errorMessage);
            }

        }


        

        private void numManualControl_ValueChanged(object sender, EventArgs e)
        {

            txtControllerValue.Text = numManualControl.Value.ToString();

        }


        #endregion
    }


}
