﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

using Usn.Control;

namespace ControlExample
{
    public partial class Form1 : Form
    {

        //Pi Controller
        Pid pid = new Pid
        {
            Ts = 0.1,
            r = 5,
            Kp = 0.8,
            Ti = 15
        };

        //Lowpassfilter
        Filter filter = new Filter
        {
            Ts = 0.1,
            Tf = 2
        };


        //Water Tank Model
        ProcessModel model = new ProcessModel(0.1);

        double levelMeasurement;
        double controllerOutput;


        public Form1()
        {
            InitializeComponent();


            timer1.Start();
            timer1.Interval = 1000;


            chartMeasurementData.Series.Clear();
            chartMeasurementData.Series.Add("MeasurementData");
            chartMeasurementData.Series["MeasurementData"].ChartType = SeriesChartType.Line;

            chartMeasurementData.Series.Add("ReferenceData");
            chartMeasurementData.Series["ReferenceData"].ChartType = SeriesChartType.Line;


            ChartArea area1 = chartMeasurementData.ChartAreas[0];
            area1.AxisY.Minimum = 0;
            area1.AxisY.Maximum = 20;


            chartController.Series.Clear();
            chartController.Series.Add("ControllerData");
            chartController.Series["ControllerData"].ChartType = SeriesChartType.Line;

            ChartArea area2 = chartController.ChartAreas[0];
            area2.AxisY.Minimum = 0;
            area2.AxisY.Maximum = 5;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {

            ProcessData();

            ControlSystem();

            Visualization();
        }


        private void ProcessData()
        {

            //Run Model
            levelMeasurement = model.LevelTankModel(controllerOutput);

            //Show Data in GUI
            txtLevelValue.Text = levelMeasurement.ToString("0.00");

        }


        private void ControlSystem()
        {



                controllerOutput = pid.PiController(levelMeasurement);

                //Scaling
                //controllerOutput = controllerOutput / 4; //0-20cm -> 0-5V

                //Set boundaries
                if (controllerOutput < 0)
                    controllerOutput = 0;
                if (controllerOutput > 5)
                    controllerOutput = 5;



            //Show Data in GUI
            txtControllerValue.Text = controllerOutput.ToString("0.00");


        }
    }
}
