﻿using System;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;
using System.Configuration;
using NationalInstruments.DAQmx;
using System.Windows.Forms.DataVisualization.Charting;



namespace Save_Data_to_Database
{
    public partial class Form1 : Form
    {
        string connectionString = ConfigurationManager.ConnectionStrings["DatabaseConnectionString"].ConnectionString;

        double measurementValue;

        public Form1()
        {
            InitializeComponent();

            timer1.Interval = 10000;
            timer1.Start();


            chartMeasurementData.Series.Clear();
            chartMeasurementData.Series.Add("MeasurementData");
            chartMeasurementData.Series["MeasurementData"].ChartType = SeriesChartType.Line;

            ChartArea area1 = chartMeasurementData.ChartAreas[0];
            area1.AxisY.Minimum = 18;
            area1.AxisY.Maximum = 25;
   
        }


        private void timer1_Tick(object sender, EventArgs e)
        {

            GetSensorData();

            DateTime timeStamp = DateTime.Now;
            txtTimeStamp.Text = timeStamp.ToString();

            Visualization();

            SaveMeasurementData();

        }

        private void txtDataRate_TextChanged(object sender, EventArgs e)
        {

            int dataRate;

            dataRate = Convert.ToInt32(txtDataRate.Text)*1000;

            timer1.Interval = dataRate;

        }


        private void GetSensorData()
        {
            Task temperatureTask = new Task();

            AIChannel myAIChannel;

            myAIChannel = temperatureTask.AIChannels.CreateThermocoupleChannel(
                "Dev1/ai0",
                "Temperature",
                0,
                100,
                AIThermocoupleType.J,
                AITemperatureUnits.DegreesC,
                25
                );


            AnalogSingleChannelReader reader = new AnalogSingleChannelReader(temperatureTask.Stream);

            double analogDataIn = reader.ReadSingleSample();

            txtMeasurementValue.Text = analogDataIn.ToString("0.0");

            measurementValue = analogDataIn;
        }

        private void SaveMeasurementData()
        {
            string sensorName;
            double measurementValue;

            sensorName = txtSensorName.Text;
            measurementValue = Convert.ToDouble(txtMeasurementValue.Text);

            try
            {
                using (SqlConnection con = new SqlConnection(connectionString))
                {
                    SqlCommand cmd = new SqlCommand("SaveMeasurementData", con);
                    cmd.CommandType = CommandType.StoredProcedure;

                    cmd.Parameters.Add(new SqlParameter("@SensorName", sensorName));
                    cmd.Parameters.Add(new SqlParameter("@MeasurementValue", measurementValue));
            
                    con.Open();
                    cmd.ExecuteNonQuery();
                    con.Close();
                }

            }
            catch (Exception ex)
            {
                throw ex;
            }

        }

        private void Visualization()
        {

            //Plot Data
            chartMeasurementData.Series["MeasurementData"].Points.AddY(measurementValue);

        }

    }
}
